const path = require('path')
const expect = require('chai').expect
const sinon = require('sinon')
const mockRequire = require('mock-require')
const Bombastic = require('../../lib/app')

const TEST_PROJECT_ROOT = path.join(__dirname, '../resources')

describe('Bombastic', function () {
  let app
  beforeEach(function () {
    app = new Bombastic(TEST_PROJECT_ROOT)
  })

  afterEach(function () {
    mockRequire.stopAll()
  })

  describe('#run()', function () {
    it('should run the correct command, passing in the app and provided options', async function () {
      const command = mockCommand('custom-command')

      await app.run('custom-command', { a: 1, b: 2 })

      expect(command.calledWith(app, { a: 1, b: 2 })).to.equal(true)
    })

    function mockCommand(commandName) {
      const moduleName = path.join(__dirname, '../../lib/', commandName, 'command')

      const mockModuleFunction = sinon.stub().resolves()
      mockRequire(moduleName, mockModuleFunction)
      return mockModuleFunction
    }
  })

  describe('#command()', function () {
    it('should return the command generator function in the lib directory by name', function () {
      const command = app.command('serve')

      expect(command.name).to.equal('serve')
      expect(command).to.be.instanceof(Function)
      expect(command.allowedOptions).to.deep.equal({
        _: [{ name: 'framework', type: 'string' }],
        port: 'int',
        useSsl: 'bool',
      })
    })

    it('should not allow requiring files outside of the bombastic/lib directory', function () {
      expect(function () {
        app.command('../test/resources')
      }).to.throw(Error)
    })
  })

  describe('emitter functions', function () {
    let events
    beforeEach(function () {
      events = []
      app.on('log', function (...args) {
        events.push(['log', args])
      })

      app.on('waiting', function (...args) {
        events.push(['waiting', args])
      })
    })

    it('should emit the correct events when certain methods are called', function () {
      app.log('info', 1, 2, 3)
      app.startWaiting()
      app.stopWaiting()

      expect(events).to.deep.equal([
        ['log', ['info', 1, 2, 3]],
        ['waiting', [true]],
        ['waiting', [false]],
      ])
    })
  })

  describe('#swagger()', function () {
    it('should load the swagger.yaml file in the root project dir', async function () {
      const spec = await app.swagger()

      expect(spec).to.be.a('object')
    })
  })

  describe('#config()', function () {
    it('should load the default bombastic.js file in the root project dir', function () {
      const config = app.config()
      expect(config).to.deep.equal({
        'value-1': 1,
        'value-2': [
          { a: 2 },
          { b: 3 },
        ],
      })
    })

    it('should load a custom file if specified by #setCustomConfigPath()', function () {
      app.setCustomConfigPath(path.join(TEST_PROJECT_ROOT, 'custom-bombastic'))

      const config = app.config()
      expect(config).to.deep.equal({
        deploy: {
          'my-deploy-target': {
            prop3: 'value3',
            prop4: 4,
          },
        },
      })
    })

    it('should not fail if the default config file cannot be found', function () {
      const newApp = new Bombastic(path.join(TEST_PROJECT_ROOT, 'handlers'))

      const config = newApp.config()
      expect(config).to.deep.equal({})
    })

    it('should not fail if a custom file is specified and is not found', function () {
      app.setCustomConfigPath(path.join(TEST_PROJECT_ROOT, 'non-existant-module'))

      const config = app.config()
      expect(config).to.deep.equal({})
    })
  })
})
