const intercept = require('intercept-stdout')
const expect = require('chai').expect
const Bombastic = require('../../../lib/app')
const CliHarness = require('../../../lib/cli/harness')

describe('CliHarness', function () {
  let app
  beforeEach(function () {
    app = new Bombastic()
  })

  let harness
  beforeEach(function () {
    harness = new CliHarness(app)
  })

  let capturedText
  let unhookIntercept
  function captureText() {
    capturedText = ''
    unhookIntercept = intercept(function (str) {
      capturedText += str
      return ''
    })
  }

  describe('#run()', function () {
    let capturedCommandName
    let capturedCommandOptions
    beforeEach(function () {
      capturedCommandName = ''
      capturedCommandOptions = {}

      // mock run function for our purposes
      app.run = function (command, options) {
        capturedCommandName = command
        capturedCommandOptions = options
      }

      // mock the command function to return the allowedOptions we want for tests
      app.command = function (command) {
        if (command === 'no-allowed-options') {
          return {}
        } else if (command === 'all-allowed-options') {
          return {
            allowedOptions: {
              _: [
                { name: 'arg1', type: 'string' },
                { name: 'arg2', type: 'int' },
                { name: 'arg3', type(str) { return `prefix_${str}` } },
              ],
              option1: 'string',
              option2: 'int',
              option3: 'bool',
              option4: 'float',
              option5(str) {
                return `${str}_suffix`
              },
              option6: false,
              option7: ['int'],
              option8: ['string'],
              option9: ['bool'],
              option10: ['int'],
            },
          }
        }

        throw new Error(`unknown command ${command}`)
      }
    })

    it('should use the first argument as the name of the command to run', function () {
      harness.run(['no-allowed-options', 'some arg', '--whatever'])

      expect(capturedCommandName).to.equal('no-allowed-options')
    })

    it('should correctly parse command line arguments based on a command\'s allowedOptions object', function () {
      harness.run([
        'all-allowed-options',
        '--option1=abcd',
        '--option2=456',
        '--option8=val1',
        '--option8=val2',
        '--option3',
        '--option7=2',
        'arg1value',
        '--option4=56.67',
        '--option7=5',
        '--option9',
        '100',
        '--option9=aaa',
        '--option5=ghij',
        'lastargvalue',
        '--option7=4',
        '--option6=somevalue',
      ])

      expect(capturedCommandOptions).to.deep.equal({
        arg1: 'arg1value',
        arg2: 100,
        arg3: 'prefix_lastargvalue',
        option1: 'abcd',
        option2: 456,
        option3: true,
        option4: 56.67,
        option5: 'ghij_suffix',
        option6: 'somevalue',
        option7: [2, 5, 4],
        option8: ['val1', 'val2'],

        // sort of pointless to test, but whatever. also note: minimist has a quirk where
        // if an option w/o a value is seen, and the option is seen again w/ a value, the
        // value will override the 'true' value from the first appearance.
        option9: [true],

        option10: [], // test when no option specified for array value
        override: [],
      })
    })
  })

  describe('#onLog()', function () {
    it('should log to the console when the "log" event is emitted by the app', function () {
      const EXPECTED_OUTPUT = `\u001b[32m[INFO]\u001b[39m this is an info message 5 = 5
\u001b[43m\u001b[37m\u001b[1m[WARN]\u001b[22m\u001b[39m\u001b[49m this is a warning obj =  { a: 1, b: 2 }
[DEBUG] this is a debug message array =  [ 1, 2, 3 ]
\u001b[41m\u001b[37m\u001b[1m[ERROR]\u001b[22m\u001b[39m\u001b[49m this is an error dummy error
`

      captureText()
      try {
        app.log('info', 'this is an info message', '5 =', 5)
        app.log('warn', 'this is a warning', 'obj = ', { a: 1, b: 2 })
        app.log('debug', 'this is a debug message', 'array = ', [1, 2, 3])
        app.log('error', 'this is an error', 'dummy error')
      } finally {
        unhookIntercept()
      }

      expect(capturedText).to.equal(EXPECTED_OUTPUT)
    })
  })

  describe('#onWaiting()', function () {
    it('should start/stop the spinner when the "waiting" event is emitted by the app', function () {
      expect(harness.spinner.isSpinning()).to.be.false

      app.startWaiting()

      expect(harness.spinner.isSpinning()).to.be.true

      app.stopWaiting()

      expect(harness.spinner.isSpinning()).to.be.false
    })
  })
})
