const path = require('path')
const expect = require('chai').expect
const RouteHandler = require('../../../lib/swagger/route-handler')

describe('RouteHandler', function () {
  describe('#constructor()', function () {
    it('should successfully parse a correct route handler specification', function () {
      const instance = new RouteHandler('export@api/reports/export')

      expect(instance.method).to.equal('export')
      expect(instance.path).to.equal('api/reports/export')
    })

    it('should fail if the method/path cannot be deduced', function () {
      expect(function () {
        return new RouteHandler('lskjfd0924lsf')
      }).to.throw(Error, /invalid route handler/i)
    })

    it('should not fail if the method is an empty string', function () {
      const instance = new RouteHandler('@api/reports/export')

      expect(instance.method).to.equal('')
      expect(instance.path).to.equal('api/reports/export')
    })

    it('should fail if the path is an empty string', function () {
      expect(function () {
        return new RouteHandler('getAll@')
      }).to.throw(Error, /invalid route handler/i)
    })

    it('should sanitize relative paths correctly to protect against require() injection', function () {
      const instance = new RouteHandler('export@../this/and/./../that')

      expect(instance.method).to.equal('export')
      expect(instance.path).to.equal('/this/and///that')
    })
  })

  describe('#getActual()', function () {
    const TEST_BASE_PATH = path.join(__dirname, '../../resources')

    it('should fail if the module cannot be require\'d', function () {
      const handler = new RouteHandler('method@non/existant/module')

      expect(function () {
        handler.getActual(TEST_BASE_PATH)
      }).to.throw(Error, /cannot find module/i)
    })

    it('should fail if require-ing a module outside of the base path is attempted', function () {
      const handler = new RouteHandler('Bombastic@../../lib')

      expect(function () {
        handler.getActual(TEST_BASE_PATH)
      }).to.throw(Error, /cannot find module/i)
    })

    it('should fail if require-ing a module provided by node_modules', function () {
      const handler = new RouteHandler('expect@chai')

      expect(function () {
        handler.getActual(TEST_BASE_PATH)
      }).to.throw(Error, /cannot find module/i)
    })

    it('should fail if the method cannot be found in the require\'d module', function () {
      const handler = new RouteHandler('mainHandler@handlers/dir/handlers')

      expect(function () {
        handler.getActual(TEST_BASE_PATH)
      }).to.throw(Error, /cannot find route handler method/i)
    })

    it('should fail if the method in the module is not a Function', function () {
      const handler = new RouteHandler('nonFunction@handlers/dir/handlers')

      expect(function () {
        handler.getActual(TEST_BASE_PATH)
      }).to.throw(Error, /route handler.*?is not a Function/i)
    })

    it('should get the default export if the route handler method is absent', function () {
      const handler = new RouteHandler('@handlers/dir/handlers')

      const func = handler.getActual(TEST_BASE_PATH)

      expect(func).is.instanceof(Function)
      expect(func.name).to.equal('mainHandler')
    })

    it('should get the named export if the route handler methid is present', function () {
      const handler = new RouteHandler('namedHandler@handlers/dir/handlers')

      const func = handler.getActual(TEST_BASE_PATH)

      expect(func).is.instanceof(Function)
      expect(func.name).to.equal('namedHandler')
    })
  })
})
